@@ -18,21 +18,21 @@ import ssl
 import subprocess
 import sys
 import tarfile
-import urllib
+import urllib.request
 from os import path
-from urllib.request import urlopen
+import platform
 
 # Needed for the GitHub Actions macOS CI runner, which appears to come without CAs.
 import certifi
 
 FREETYPE_HOST = "https://download.savannah.gnu.org/releases/freetype/"
-FREETYPE_TARBALL = "freetype-2.10.2.tar.xz"
+FREETYPE_TARBALL = "freetype-2.12.0.tar.xz"
 FREETYPE_URL = FREETYPE_HOST + FREETYPE_TARBALL
-FREETYPE_SHA256 = "1543d61025d2e6312e0a1c563652555f17378a204a61e99928c9fcef030a2d8b"
-HARFBUZZ_HOST = "https://www.freedesktop.org/software/harfbuzz/release/"
-HARFBUZZ_TARBALL = "harfbuzz-2.6.7.tar.xz"
+FREETYPE_SHA256 = "ef5c336aacc1a079ff9262d6308d6c2a066dd4d2a905301c4adda9b354399033"
+HARFBUZZ_HOST = "https://github.com/harfbuzz/harfbuzz/releases/download/4.2.1/"
+HARFBUZZ_TARBALL = "harfbuzz-4.2.1.tar.xz"
 HARFBUZZ_URL = HARFBUZZ_HOST + HARFBUZZ_TARBALL
-HARFBUZZ_SHA256 = "49e481d06cdff97bf68d99fa26bdf785331f411614485d892ea4c78eb479b218"
+HARFBUZZ_SHA256 = "bd17916513829aeff961359a5ccebba6de2f4bf37a91faee3ac29c120e3d7ee1"
 
 root_dir = "."
 build_dir = path.join(root_dir, "build")
@@ -70,7 +70,7 @@ if sys.platform == "win32":
 if sys.platform == "darwin":
     print("# Making a 64 bit build.")
     CMAKE_GLOBAL_SWITCHES += (
-        '-DCMAKE_OSX_ARCHITECTURES="x86_64" '
+        '-DCMAKE_OSX_ARCHITECTURES="x86_64;arm64" '
         '-DCMAKE_OSX_DEPLOYMENT_TARGET="10.9" '
         '-DCMAKE_C_FLAGS="-O2" '
         '-DCMAKE_CXX_FLAGS="-O2" '
@@ -78,27 +78,37 @@ if sys.platform == "darwin":
     bitness = 64
 
 if "linux" in sys.platform:
-    if (
-        os.environ.get("PYTHON_ARCH", "") == "32"
-        or os.environ.get("PLAT", "") == "i686"
-    ):
-        print("# Making a 32 bit build.")
-        # On a 64 bit Debian/Ubuntu, this needs gcc-multilib and g++-multilib.
-        # On a 64 bit Fedora, install glibc-devel and libstdc++-devel.
-        CMAKE_GLOBAL_SWITCHES += (
-            '-DCMAKE_C_FLAGS="-m32 -O2" '
-            '-DCMAKE_CXX_FLAGS="-m32 -O2" '
-            '-DCMAKE_LD_FLAGS="-m32" '
-        )
-        bitness = 32
+    c_flags = cxx_flags = "-O2"
+    ld_flags = ""
+    if platform.machine() == "x86_64":
+        if (
+            os.environ.get("PYTHON_ARCH", "") == "32"
+            or platform.architecture() == "32bit"
+        ):
+            print("# Making a 32 bit build.")
+            # On a 64 bit Debian/Ubuntu, this needs gcc-multilib and g++-multilib.
+            # On a 64 bit Fedora, install glibc-devel and libstdc++-devel.
+            c_flags = "-m32 {}".format(c_flags)
+            cxx_flags = "-m32 {}".format(cxx_flags)
+            ld_flags = "-m32"
+            bitness = 32
+        else:
+            print("# Making a 64 bit build.")
+            c_flags = "-m64 {}".format(c_flags)
+            cxx_flags = "-m64 {}".format(cxx_flags)
+            ld_flags = "-m64"
+            bitness = 64
     else:
-        print("# Making a 64 bit build.")
-        CMAKE_GLOBAL_SWITCHES += (
-            '-DCMAKE_C_FLAGS="-m64 -O2" '
-            '-DCMAKE_CXX_FLAGS="-m64 -O2" '
-            '-DCMAKE_LD_FLAGS="-m64" '
-        )
-        bitness = 64
+        print("# Making a '{}' build.".format(platform.machine()))
+        if platform.architecture() == "32bit":
+            bitness = 32
+        else:
+            bitness = 64
+    CMAKE_GLOBAL_SWITCHES += (
+        '-DCMAKE_C_FLAGS="{}" '.format(c_flags) +
+        '-DCMAKE_CXX_FLAGS="{}" '.format(cxx_flags) +
+        '-DCMAKE_LD_FLAGS="{}" '.format(ld_flags)
+    )
 
 
 def shell(cmd, cwd=None):
